package cn.com.ctop.kuaishou.modules.batch.controller;


import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouCampaign;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouGroup;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouRegionListParent;
import cn.com.ctop.kuaishou.modules.batch.entity.vo.SpendVo;
import cn.com.ctop.kuaishou.modules.batch.mapper.KuaiShouRegionListParentMapper;
import cn.com.ctop.kuaishou.modules.batch.service.*;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.List;

@Slf4j
@Api(tags = "快手-批量工具")
@RestController
@RequestMapping("/kuaishou/batch")
public class BatchController {
    @Autowired
    private ICtopOauthTokenService oauthTokenService;
    @Autowired
    private IKuaishouInterfaceService iKuaishouInterfaceService;
    @Autowired
    private IBatchService batchService;
    @Autowired
    private IKuaiShouCampaignService kuaiShouCampaignService;
    @Autowired
    private IKuaiShouUpdateService updateService;
    @Autowired
    private ICtopOauthTokenService tokenService;

    @Autowired
    private IKuaiShouGroupService kuaiShouGroupService;
    @Autowired
    private KuaiShouRegionListParentMapper regionListParentMapper;


    /**
     * 获取花费信息
     *
     * @param accountId
     * @return
     */
    @GetMapping(value = "/spend")
    public Result<SpendVo> spend(Long accountId) {
        Result<SpendVo> result = new Result<>();
        try {

            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
            if (Check.isNull(oauthToken)) {
                throw new Exception("未获取到账户信息");
            }
            SpendVo spendVo = new SpendVo();
            JSONObject fundJson = iKuaishouInterfaceService.fundGet(oauthToken);
            if (!Check.isNull(fundJson)) {
                spendVo.setBalance(fundJson.getBigDecimal("balance")); // 余额
            }
            BigDecimal cost = batchService.getCost(accountId);
            spendVo.setCost(cost);

            BigDecimal budget = batchService.getBudget(accountId);
            spendVo.setBudget(budget);
            result.setSuccess(true);
            result.setResult(spendVo);

        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }

        return result;
    }


    /**
     * 获取广告计划列表
     *
     * @param kuaiShouCampaign
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/getCampaignList")
    public Result<IPage<KuaiShouCampaign>> getCampaignList(KuaiShouCampaign kuaiShouCampaign,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        Result<IPage<KuaiShouCampaign>> result = new Result<IPage<KuaiShouCampaign>>();
        QueryWrapper<KuaiShouCampaign> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouCampaign, req.getParameterMap());
        queryWrapper.orderByDesc("put_create_time");
        Page<KuaiShouCampaign> page = new Page<KuaiShouCampaign>(pageNo, pageSize);
        IPage<KuaiShouCampaign> pageList = kuaiShouCampaignService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    /**
     * 批量修改广告计划预算
     *
     * @param requestJson
     * @return
     */
    @PostMapping(value = "/batchUpdateDayBudget")
    public Result<JSONObject> batchUpdateDayBudget(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();

        try {
            Long accountId = requestJson.getLong("accountId");
            CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
            if (Check.isNull(token)) {
                throw new Exception("账号信息为空");
            }
            Long dayBudget = requestJson.getLong("dayBudget");
            if (Check.isNull(dayBudget)) {
                throw new Exception("预算信息为空");
            }
            String userId = requestJson.getString("userId");
            JSONArray campaignIds = requestJson.getJSONArray("campaignIds");
            if (!Check.isNull(campaignIds)) {
                for (int i = 0; i < campaignIds.size(); i++) {
                    Long campaignId = campaignIds.getLong(i);
                    if (!Check.isNull(campaignId)) {
                        updateService.updateCampaign(token.getAccessToken(), accountId, campaignId, dayBudget, userId);
                    }
                }
            }
            result.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    /**
     * 查询广告组列表
     *
     * @param kuaiShouGroup
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/getUnitList")
    public Result<IPage<KuaiShouGroup>> getUnitList(KuaiShouGroup kuaiShouGroup,
                                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                    HttpServletRequest req) {
        Result<IPage<KuaiShouGroup>> result = new Result<IPage<KuaiShouGroup>>();
        QueryWrapper<KuaiShouGroup> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouGroup, req.getParameterMap());
        Page<KuaiShouGroup> page = new Page<KuaiShouGroup>(pageNo, pageSize);
        IPage<KuaiShouGroup> pageList = kuaiShouGroupService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    /**
     * 获取区域列表
     *
     * @param kuaiShouRegionListParent
     * @return
     */
    @GetMapping(value = "/getRegion")
    public Result<List<KuaiShouRegionListParent>> getRegion(KuaiShouRegionListParent kuaiShouRegionListParent, HttpServletRequest req) {
        Result<List<KuaiShouRegionListParent>> result = new Result<>();

        try {
            QueryWrapper<KuaiShouRegionListParent> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouRegionListParent, req.getParameterMap());

            List<KuaiShouRegionListParent> regionListParents = regionListParentMapper.selectList(queryWrapper);
            result.setSuccess(true);
            result.setResult(regionListParents);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);

        }
        return result;
    }

}
